@@ -1,7 +1,7 @@
 /* telepen.c - Handles Telepen and Telepen numeric */
 /*
     libzint - the open source barcode library
-    Copyright (C) 2008-2024 Robin Stuart <rstuart114@gmail.com>
+    Copyright (C) 2008-2025 Robin Stuart <rstuart114@gmail.com>
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -86,27 +86,28 @@ static const char TeleLens[128] = {
     12, 10,  8, 14, 10, 12, 12, 12, 10, 14, 12, 12, 14, 12, 12, 16
 };
 
-INTERNAL int telepen(struct zint_symbol *symbol, unsigned char source[], int length) {
+INTERNAL int zint_telepen(struct zint_symbol *symbol, unsigned char source[], int length) {
     int i, count, check_digit;
     int error_number;
     char dest[1145]; /* 12 (Start) + 69 * 16 (max for DELs) + 16 (Check) + 12 (stop) + 1 = 1145 */
     char *d = dest;
+    const int content_segs = symbol->output_options & BARCODE_CONTENT_SEGS;
 
     error_number = 0;
 
     count = 0;
 
     if (length > 69) { /* 16 (Start) + 69 * 16 + 16 (Check) + 16 (Stop) = 1152 */
-        return errtxtf(ZINT_ERROR_TOO_LONG, symbol, 390, "Input length %d too long (maximum 69)", length);
+        return z_errtxtf(ZINT_ERROR_TOO_LONG, symbol, 390, "Input length %d too long (maximum 69)", length);
     }
     /* Start character */
     memcpy(d, TeleTable['_'], 12);
     d += 12;
 
     for (i = 0; i < length; i++) {
-        if (source[i] > 127) {
+        if (!z_isascii(source[i])) {
             /* Cannot encode extended ASCII */
-            return errtxtf(ZINT_ERROR_INVALID_DATA, symbol, 391,
+            return z_errtxtf(ZINT_ERROR_INVALID_DATA, symbol, 391,
                             "Invalid character at position %d in input, extended ASCII not allowed", i + 1);
         }
         memcpy(d, TeleTable[source[i]], TeleLens[source[i]]);
@@ -127,70 +128,68 @@ INTERNAL int telepen(struct zint_symbol *symbol, unsigned char source[], int len
     memcpy(d, TeleTable['z'], 12);
     d += 12;
 
-    expand(symbol, dest, d - dest);
+    z_expand(symbol, dest, d - dest);
 
     if (symbol->output_options & COMPLIANT_HEIGHT) {
         /* Default height from various Telepen docs is based on default 26pt at X 0.01125"
            (average of 0.01" - 0.0125") = (26 / 72) / 0.01125 ~ 32; no min height specified */
-        (void) set_height(symbol, 0.0f, 32.0f, 0, 1 /*no_errtxt*/);
+        (void) z_set_height(symbol, 0.0f, 32.0f, 0, 1 /*no_errtxt*/);
     } else {
-        (void) set_height(symbol, 0.0f, 50.0f, 0, 1 /*no_errtxt*/);
+        (void) z_set_height(symbol, 0.0f, 50.0f, 0, 1 /*no_errtxt*/);
     }
 
-    for (i = 0; i < length; i++) {
-        if (source[i] == '\0') {
-            symbol->text[i] = ' ';
-        } else {
-            symbol->text[i] = source[i];
-        }
+    z_hrt_cpy_iso8859_1(symbol, source, length);
+
+    if (content_segs && z_ct_cpy_cat(symbol, source, length, (char) check_digit, NULL /*cat*/, 0)) {
+        return ZINT_ERROR_MEMORY; /* `z_ct_cpy_cat()` only fails with OOM */
     }
-    symbol->text[length] = '\0';
+
     return error_number;
 }
 
-INTERNAL int telepen_num(struct zint_symbol *symbol, unsigned char source[], int length) {
+INTERNAL int zint_telepen_num(struct zint_symbol *symbol, unsigned char source[], int length) {
     int count, check_digit, glyph;
     int error_number = 0;
     int i;
     char dest[1129]; /* 12 (Start) + 68 * 16 (max for DELs) + 16 (Check) + 12 (Stop) + 1 = 1129 */
     char *d = dest;
-    unsigned char temp[137];
+    unsigned char local_source[137];
+    const int content_segs = symbol->output_options & BARCODE_CONTENT_SEGS;
 
     count = 0;
 
     if (length > 136) { /* 68*2 */
-        return errtxtf(ZINT_ERROR_TOO_LONG, symbol, 392, "Input length %d too long (maximum 136)", length);
+        return z_errtxtf(ZINT_ERROR_TOO_LONG, symbol, 392, "Input length %d too long (maximum 136)", length);
     }
-    if ((i = not_sane(SODIUM_X_F, source, length))) {
-        return errtxtf(ZINT_ERROR_INVALID_DATA, symbol, 393,
+    if ((i = z_not_sane(SODIUM_X_F, source, length))) {
+        return z_errtxtf(ZINT_ERROR_INVALID_DATA, symbol, 393,
                         "Invalid character at position %d in input (digits and \"X\" only)", i);
     }
 
     /* Add a leading zero if required */
     if (length & 1) {
-        memcpy(temp + 1, source, length++);
-        temp[0] = '0';
+        memcpy(local_source + 1, source, length++);
+        local_source[0] = '0';
     } else {
-        memcpy(temp, source, length);
+        memcpy(local_source, source, length);
     }
-    temp[length] = '\0';
-    to_upper(temp, length);
+    z_to_upper(local_source, length);
 
     /* Start character */
     memcpy(d, TeleTable['_'], 12);
     d += 12;
 
     for (i = 0; i < length; i += 2) {
-        if (temp[i] == 'X') {
-            return errtxtf(ZINT_ERROR_INVALID_DATA, symbol, 394, "Invalid odd position %d of \"X\" in Telepen data",
+        if (local_source[i] == 'X') {
+            return z_errtxtf(ZINT_ERROR_INVALID_DATA, symbol, 394, "Invalid odd position %d of \"X\" in Telepen data",
                             i + 1);
         }
 
-        if (temp[i + 1] == 'X') {
-            glyph = ctoi(temp[i]) + 17;
+        if (local_source[i + 1] == 'X') {
+            glyph = z_ctoi(local_source[i]) + 17;
             count += glyph;
         } else {
-            glyph = (10 * ctoi(temp[i])) + ctoi(temp[i + 1]);
+            glyph = 10 * z_ctoi(local_source[i]) + z_ctoi(local_source[i + 1]);
             glyph += 27;
             count += glyph;
         }
@@ -211,15 +210,20 @@ INTERNAL int telepen_num(struct zint_symbol *symbol, unsigned char source[], int
     memcpy(d, TeleTable['z'], 12);
     d += 12;
 
-    expand(symbol, dest, d - dest);
+    z_expand(symbol, dest, d - dest);
 
     if (symbol->output_options & COMPLIANT_HEIGHT) {
-        (void) set_height(symbol, 0.0f, 32.0f, 0, 1 /*no_errtxt*/); /* Same as alphanumeric Telepen */
+        (void) z_set_height(symbol, 0.0f, 32.0f, 0, 1 /*no_errtxt*/); /* Same as alphanumeric Telepen */
     } else {
-        (void) set_height(symbol, 0.0f, 50.0f, 0, 1 /*no_errtxt*/);
+        (void) z_set_height(symbol, 0.0f, 50.0f, 0, 1 /*no_errtxt*/);
+    }
+
+    z_hrt_cpy_nochk(symbol, local_source, length);
+
+    if (content_segs && z_ct_cpy_cat(symbol, local_source, length, (char) check_digit, NULL /*cat*/, 0)) {
+        return ZINT_ERROR_MEMORY; /* `z_ct_cpy_cat()` only fails with OOM */
     }
 
-    ustrcpy(symbol->text, temp);
     return error_number;
 }
 
